<?php
/*
 * Created on 19.12.2022 by Nico Schubert
 * Terminverwaltung Version 1.31 (https://www.php-space.info/terminverwaltungs-script/)
 * Copyright by Php-Space.info
 * Web: www.php-space.info
 * Support Forum: http://ww.php-space.info/forum/
 * E-Mail: info@schubertmedia.de
 */

class DBI {
	public $DBD;
	public $CONN;
	public $CONFIG;

	public $DBTYPE;

	public $prepare = "";
	public $row     = array();
  /**
   * @var string #[\SensitiveParameter]
   */
  public $dbsocket=false;
  /**
   * @var string #[\SensitiveParameter]
   */
  public string $dbhost;
  /**
   * @var string #[\SensitiveParameter]
   */
  public string $dbname;
  /**
   * @var string #[\SensitiveParameter]
   */
  public string $dbuser;
  /**
   * @var string #[\SensitiveParameter]
   */
  public string $dbpwd;
  /**
   * @var string
   */
  protected string $cachedir;
  public int $dbport=3306;


  #[NoReturn] function error($str): void
  {
		echo "$str<BR>\n";
		exit;

	}

  /**
   * @param string $datenbank_host
   * @param string $datenbank_name
   * @param string $datenbank_user
   * @param string $datenbank_password
   * @param string $cachedir
   */
  function __construct(#[\SensitiveParameter] string $datenbank_host, #[\SensitiveParameter] string $datenbank_name, #[\SensitiveParameter] string $datenbank_user, #[\SensitiveParameter] string $datenbank_password, string $cachedir)
	{
		$this->dbhost 	= $datenbank_host;
		$this->dbname 	= $datenbank_name;
		$this->dbuser 	= $datenbank_user;
		$this->dbpwd  	= $datenbank_password;
		$this->cachedir = $cachedir;
		$this->DBD = new DBD_mysqli($this);
	}

	function connect(): true
  {
		if( $this->DBTYPE == 'mysql' )
		{
			$this->CONN = $this->DBD->connect(
				$this->dbname,
				( ( strlen( $this->dbsocket ) ) ? ":". $this->dbsocket : $this->dbhost.":".$this->dbport ),
				$this->dbuser,
				$this->dbpwd
			);
		}
		else {
			$this->CONN = $this->DBD->connect(
				$this->dbname,
				$this->dbhost,
				$this->dbuser,
				$this->dbpwd,
				$this->dbport,
				( strlen( $this->dbsocket ) ? $this->dbsocket : false )
			);
		}

		return true;

	} // End Connect

	function disconnect(): void
  {
		if($this->DBD){
			$this->DBD->disconnect();
		}
	} // End Disconnect

	function prepare($query,$cache = 0)
	{
		$sth = $this->DBD->prepare($query, $cache);
		return($sth);

	} // End Prepare

}	// End Class

class DBD_mysqli {

	public $CONFIG;
	public $dbh;

	function __construct(DBI $pConfig)
	{
		if ($pConfig !== false) {
			$this->CONFIG = $pConfig;
		}
	}

	function error($str)
	{

		echo ($str);

	} // End Error


  /**
   * @return DBD_mysqli|void
   */
  function connect(){

    try {
      if ($this->CONFIG->dbsocket === false)
        $conn = mysqli_connect($this->CONFIG->dbhost, $this->CONFIG->dbuser, $this->CONFIG->dbpwd, $this->CONFIG->dbname, $this->CONFIG->dbport);
      else
        $conn = mysqli_connect($this->CONFIG->dbhost, $this->CONFIG->dbuser, $this->CONFIG->dbpwd, $this->CONFIG->dbname, $this->CONFIG->dbport, $this->CONFIG->dbsocket);
    } catch (Exception $e) {
      $this->error('DBD::mysqli Error: '.$e->getMessage());
      exit();
    }

		if(!$conn) {
			$this->error("DBD::mysql Error: Could not connect to the database.");
			return;
		}

		$this->dbh = $conn;

		return($this);

	} // End Connect

	function disconnect( ): int
  {
		if($this->dbh){
			mysqli_close($this->dbh);
		}
		unset( $this->dbh );

		return(1);

	} // End Disconnect

  /**
   * @param string $query
   * @param int $cache
   * @return STH_mysqli
   */
  function prepare(string $query, int $cache = 0): STH_mysqli
  {
		$sth = new STH_mysqli;
		$sth->DBH($this);
		$sth->prepare($query, $cache);
		return($sth);

	}

  function escape($string){

    if($this->dbh==''){
      $this->connect();
    }
    if($this->dbh) {
      return (mysqli_real_escape_string($this->dbh, $string));
    }
  }

} // End DBD Class

class STH_mysqli {

	public $query;
	public $DBH;
	public $dbh;
	public $result;
	public $cache;

	function error($str)
	{
		echo "$str<BR>\n";
		exit;

	} // End Error

	function DBH(&$dbh)
	{
		$this->DBH = &$dbh;
		$this->dbh = &$dbh->dbh;
		return( $dbh );
	}

	function prepare(string $query, int $cache)
	{
		if ($cache && preg_match("/^SELECT.+/", trim($query))) {
			$this->cache['sth'] = new CACHE($cache);
			$this->cache['file'] = "../cache/queries/".md5($query);
		}
		$this->query = $query;
		return($query);
	}

	function execute(): int
  {
		if(is_array($this->cache)){

			$this->cache['data']['row']['array'] = array();
			$this->cache['data']['row']['hash'] = array();
			$this->cache['data']['row']['hashref'] = array();

			if($this->cache['sth']->read_cache_file($this->cache['file'], $this->cache['data'])){

				$this->cache['data'] = unserialize(base64_decode($this->cache['data']));
				$this->result = $this->cache['data']['result'];
			}
			if( !$this->result ) {

			$this->cache['init'] = true;
				if(!$this->dbh){
					$this->DBH->connect(
						$this->DBH->CONFIG->dbname,
						$this->DBH->CONFIG->dbhost,
						$this->DBH->CONFIG->dbuser,
						$this->DBH->CONFIG->dbpwd,
						$this->DBH->CONFIG->dbport,
						( strlen( $this->DBH->CONFIG->dbsocket ) ? $this->DBH->CONFIG->dbsocket : false )
					);
				}
				$this->result = @mysqli_query($this->dbh, $this->query);
				$this->cache['data']['result'] = ( $this->result ) ? 1 : 0;
			}
		}
		else{
			if(!$this->dbh){
				$this->DBH->connect(
					$this->DBH->CONFIG->dbname,
					$this->DBH->CONFIG->dbhost,
					$this->DBH->CONFIG->dbuser,
					$this->DBH->CONFIG->dbpwd,
					$this->DBH->CONFIG->dbport,
					( strlen( $this->DBH->CONFIG->dbsocket ) ? $this->DBH->CONFIG->dbsocket : false )
				);
			}
      try {
        $this->result = mysqli_query($this->dbh, $this->query);
      } catch (Exception $e) {
        $this->error('DBD::mysqli Error: '.$e->getMessage());
        exit();
      }
    }

		if(!$this->result)
		{
			return(0);
		}

		return(1);

	} // End Execute

	function insert_id(): int|string
  {
		return(mysqli_insert_id($this->dbh));

	}
  function escape($string){
    return (mysqli_real_escape_string($this->dbh, $string));
  }


  function finish()
	{
		if(!$this->result){ return; }

		if(!is_array($this->cache) || isset($this->cache['init'])){

			mysqli_free_result($this->result);

			if(isset($this->cache['init'])){

				$this->cache['sth']->write_cache_file($this->cache['file'], $xxx = base64_encode(serialize($this->cache['data'])));
			}
		}
		unset( $this->result );

		return(1);

	} // End Finish

	function fields()
	{
		$fields = 0;
		if(!is_array($this->cache) || isset($this->cache['init'])){
			$fields = @mysqli_num_fields($this->result);
			if(isset($this->cache['init'])){
				$this->cache['data']['fields'] = $fields;
			}
		}
		elseif(isset($this->cache['data']['fields'])){
			$fields = $this->cache['data']['fields'];
		}

		return $fields;

	} // End Rows

	function rows()
	{
		$rows = 0;
		if(!is_array($this->cache) || isset($this->cache['init'])){
			$rows = mysqli_num_rows($this->result);
			if(isset($this->cache['init'])){
				$this->cache['data']['rows'] = $rows;
			}
		}
		elseif(isset($this->cache['data']['rows'])){
			$rows = $this->cache['data']['rows'];
		}
		return $rows;

	} // End Rows

	function affected()
	{
		$rows = 0;
		if(!is_array($this->cache) || isset($this->cache['init'])){
			$rows = @mysqli_affected_rows($this->dbh);
			if(isset($this->cache['init'])){
				$this->cache['data']['affected'] = $rows;
			}
		}
		elseif(isset($this->cache['data']['affected'])){
			$rows = $this->cache['data']['affected'];
		}
		return $rows;

	} // End Rows

	function fetchrow_array()
	{
		if(!$this->result){
			//echo "STH Error: Calling fetchrow on null result.<br>\n";
			return;
		}
		$row = false;
		if(!is_array($this->cache) || isset($this->cache['init'])){
			$row = mysqli_fetch_array($this->result);
			if(isset($this->cache['init'])){
				$this->cache['data']['row']['array'][] = $row;
			}
		}
		elseif(isset($this->cache['data']['row']['array'])){
			$row = array_shift($this->cache['data']['row']['array']);
		}
		return($row);

	}

	function fetchrow_hash()
	{
		if(!$this->result){
			//echo "STH Error: Calling fetchrow on null result.<br>\n";
			return;
		}
		$row = false;
		if(!is_array($this->cache) || isset($this->cache['init'])){
			$row = mysqli_fetch_assoc($this->result);
			if(isset($this->cache['init'])){
				$this->cache['data']['row']['hash'][] = $row;
			}
		}
		elseif(isset($this->cache['data']['row']['hash'])){
			$row = array_shift($this->cache['data']['row']['hash']);
		}
		return($row);

	} // End Fetchrow_hash

	function fetchrow_hashref()
	{
		if(!$this->result){
			//echo "STH Error: Calling fetchrow on null result.<br>\n";
			return;
		}
		$row = false;
		if(!is_array($this->cache) || isset($this->cache['init'])){
			$row = mysqli_fetch_object($this->result);
			if(isset($this->cache['init'])){
				$this->cache['data']['row']['hashref'][] = $row;
			}
		}
		elseif(isset($this->cache['data']['row']['hashref'])){
			$row = array_shift($this->cache['data']['row']['hashref']);
		}
		return($row);

	} // End Fetchrow_hashref

	function fetchrow_field()
	{
		if(!$this->result){
			//echo "STH Error: Calling fetchrow on null result.<br>\n";
			return;
		}
		$row = false;
		if(!is_array($this->cache) || isset($this->cache['init'])){
			$row = mysqli_fetch_field($this->result);
			if(isset($this->cache['init'])){
				$this->cache['data']['row']['field'][] = $row;
			}
		}
		elseif(isset($this->cache['data']['row']['field'])){
			$row = array_shift($this->cache['data']['row']['field']);
		}
		return($row);

	} // End Fetchrow_field

} // End Class STH

class CACHE
{
  private int $expires;

  /**
   * @param int $pExpires
   */
  function __construct(int $pExpires = 30 ) {

    $this->expires = $pExpires;
  }

  function write_cache_file ( string $filename, string $data ): bool
  {

  	$result = false;

	$fp = @fopen($filename, "w");

  	if ($fp) {
  		if (! @flock($fp, LOCK_EX | LOCK_NB))
  			@fclose($fp);
  	}

  	if ($fp) {
  		@fputs($fp, $data);
  		@fclose($fp);
  		$result = true;
  	}

  	if( $result ) {
  		if( isset($_GET['dbdebug']) ) echo "OK ($filename)";
  	}
  	else {
  		if( isset($_GET['dbdebug']) ) echo "Failed ($filename)";
  	}

  	return $result;
  }

  /**
   * @param string $filename
   * @param $data
   * @return bool
   */
  function read_cache_file (string $filename, &$data ): bool
  {

  	$result = false;

  	if ( !$this->expire_cache_file( $filename ) ) {
	  	$fp='';
  		if (@is_readable($filename))
  			$fp = @fopen($filename, "r");

  		if ($fp) {
  			if (! @flock($fp, LOCK_SH | LOCK_NB))
  				@fclose($fp);
  		}

  		if ($fp) {
  			$data = "";
  			while (! @feof($fp)) {
  				$data .= @fread($fp, 131072);
  			}
  			@fclose($fp);
  			$result = true;
  		}

  	}
  	if( $result ) {
  		if( isset($_GET['dbdebug']) ) echo "OK ($filename)";
  	}

  	return $result;
  }

  function expire_cache_file( string $filename ): bool
  {

  	$result = false;

    if (@is_readable($filename)) {
    	$mtime = @filemtime($filename);
    	$act_time = @time();

    	if ( $mtime < $act_time - $this->expires ) {
    		@unlink($filename);
    		$result = true;

    		if( isset($_GET['dbdebug']) ) echo "Expired ($filename)";
    	}
    }
    else {
    	if( isset($_GET['dbdebug']) ) echo "Not exists ($filename)";
    }
  	return $result;
  }
}